
div.dialpatterns {
	max-height: 15em; 
	max-width: 100%; 
	width: 500px;
	overflow: auto; 
	border: 1px solid #CCCCCC;
    white-space: nowrap;
}
#pattern_file {
	display: none;
}
.save, .adv_set_default{border-style: none;}
.savetd, .save{display:none;}

th { padding-top: 1em; border-bottom: 2px solid black; }
tr>td:first-child { width: 20em; }
